﻿/*
------------------------------Copyright----------------------------
 SuperMap iDesktop 8C sample code
 Copyright: SuperMap Software Co., Ltd.
------------------------------------------------------------------

-----------------SuperMap iDesktop 8C Sample Description---------------

1. Introduction: How to add a menu that you want in the existed context menu. And how to interact with the scene.
2. Operation：First, click an object in the scene. It will display a bubble. The bubble shows the detailed informatin: object type, layer, coordinates and so on.
              Then, right click the selected object, it will show："increase", "decrease", "move left", "move right".
3. Data：Workspace(SampleData/3D/CBDDataset/CBD.sxwu); Scene(CBD)
4. Key files, types and members:
    Plugin.config Configuration file
    Plugin.Initialize Method
    Plugin.ExitInstance Method
    Application.CreateSceneWindow Methos
    IFormManager.FormCreated Event
    Application.ActiveForm Property
    IFormScene.SceneControl Property
    ICtrlAction.Enable Method
    Other component interfaces

5. Steps:
   (1)Open the workspace and scene.
   (2)Select an object and the bubble shows the object information.
   (3)Select the building object. Right click Decrease and Decrease to adjust the building height.
   (4)Select an object, and right click Left, Right, Up and Down.
 
 ----------------------------------------------------------------------------
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using SuperMap.Desktop;
using SuperMap.Data;
using SuperMap.UI;
using SuperMap.Realspace;
using System.Drawing;
using System.IO;

namespace SuperMap.Desktop.Samples
{
    public class DesktopPlugin : Plugin
    {
        //private Workspace m_workspace;
        private SceneControl m_sceneControl;
        private CustomBubbleControl m_customBubbleControl;
        private Bubble m_bubble;
        private String m_currentDir = Directory.GetCurrentDirectory()+Path.DirectorySeparatorChar;
        private Boolean m_showBubble = false; 

        public DesktopPlugin(PluginInfo pluginInfo)
            : base(pluginInfo)
        {
            
        }

        public override Boolean Initialize()
        {
            try
            {
                IFormManager formManager = Application.ActiveApplication.MainForm.FormManager;
                (Application.ActiveApplication.MainForm as Form).MdiChildActivate += new EventHandler(DesktopPlugin_MdiChildActivate);
            }
            catch (Exception ex)
            {
                SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
            return true;
        }
        void DesktopPlugin_MdiChildActivate(object sender, EventArgs e)
        {
            try
            {
                //IFormScene formScene = e.Form as IFormScene;
                IFormScene formScene = Application.ActiveForm as IFormScene;
                
                //if (formScene != null && formScene.SceneControl.Scene.Name == "CBD" && m_sceneControl == null)
                //{
                if (formScene != null)
                {
                    m_sceneControl = formScene.SceneControl;
                    m_customBubbleControl = new CustomBubbleControl();
                    m_sceneControl.Controls.Add(m_customBubbleControl);
                    m_customBubbleControl.Visible = false;

                    //register events
                    m_sceneControl.ObjectSelected += new ObjectSelectedEventHandler(m_sceneControl_ObjectSelected);
                    m_sceneControl.MouseDown += new MouseEventHandler(m_sceneControl_MouseDown);
                    m_sceneControl.MouseUp += new MouseEventHandler(m_sceneControl_MouseUp);
                    m_sceneControl.BubbleInitialize += new BubbleInitializeEventHandler(m_sceneControl_BubbleInitialize);
                    m_sceneControl.BubbleClose += new BubbleCloseEventHandler(m_sceneControl_BubbleClose);
                    m_sceneControl.BubbleResize += new BubbleResizeEventHandler(m_sceneControl_BubbleResize);
                }          
            }
            catch (System.Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
        }

        /// <summary>
        /// When disable these plugins, unregister these events.
        /// </summary>
        /// <returns></returns>
        public override Boolean ExitInstance()
        {
            IFormManager formManager = Application.ActiveApplication.MainForm.FormManager;
            (Application.ActiveApplication.MainForm as Form).MdiChildActivate -= new EventHandler(DesktopPlugin_MdiChildActivate);
            //formManager.FormCreated -= new FormCreatedEventHandler(formManager_FormCreated);
            return true;
        }

        /// <summary>
        /// Register
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void formManager_FormCreated(object sender, FormCreatedEventArgs e)
        {
            try
            {
                IFormScene formScene = e.Form as IFormScene;

                //Whether it is the scene of the current sample
                //if (formScene != null && formScene.SceneControl.Scene.Name == "Symbolism Modeling")
                //{                  
                    m_sceneControl = formScene.SceneControl;
                    m_customBubbleControl = new CustomBubbleControl();
                    m_sceneControl.Controls.Add(m_customBubbleControl);
                    m_customBubbleControl.Visible = false;

                    //register events
                    m_sceneControl.ObjectSelected += new ObjectSelectedEventHandler(m_sceneControl_ObjectSelected);
                    m_sceneControl.MouseDown += new MouseEventHandler(m_sceneControl_MouseDown);
                    m_sceneControl.MouseUp += new MouseEventHandler(m_sceneControl_MouseUp);
                    m_sceneControl.BubbleInitialize += new BubbleInitializeEventHandler(m_sceneControl_BubbleInitialize);
                    m_sceneControl.BubbleClose += new BubbleCloseEventHandler(m_sceneControl_BubbleClose);
                    m_sceneControl.BubbleResize += new BubbleResizeEventHandler(m_sceneControl_BubbleResize);
                //}
            }
            catch (Exception ex)
            {
                SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
        }
        /// <summary>
        /// 选择场景中对象，弹出气泡。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void m_sceneControl_ObjectSelected(object sender, ObjectSelectedEventArgs e)
        {
            if (m_showBubble)
            {
                try
                {
                    if (m_sceneControl.Bubbles.Count > 0)
                    {
                        m_sceneControl.Bubbles.Clear();
                    }
                    Selection3D[] selection3Ds = m_sceneControl.Scene.FindSelection(true);
                    if (selection3Ds.Length == 1)
                    {
                        Selection3D selection = selection3Ds[0];
                        fillCustomBubbleControl(selection);

                        # region Create and initialize the bubble object
                        m_bubble = new Bubble();
                        m_bubble.Title = "Basic Information";// Set title for the bubble
                        m_bubble.RoundQuality = 2;// Set the rounded corner of the bubble
                        m_bubble.IsAutoHide = true;// Set whether the bubble is hide
                        m_bubble.FrameWidth = 2;// Set the border width of the bubble
                        m_bubble.ClientWidth = m_customBubbleControl.Width + 4;// Set the width of bubble area
                        m_bubble.ClientHeight = m_customBubbleControl.Height + 4;// Set the height of bubble area
                        TextStyle titleStyle = new TextStyle();// Set bubble text style
                        titleStyle.FontName = "Times New Roman";
                        titleStyle.ForeColor = Color.Green;
                        titleStyle.Bold = true;
                        m_bubble.TitleTextStyle = titleStyle;
                        m_sceneControl.Bubbles.Add(m_bubble);
                        #endregion
                    }
                }
                catch (Exception ex)
                {
                    SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
                }
            }
        }

        void m_sceneControl_MouseDown(object sender, MouseEventArgs e)
        {
            m_showBubble = true;
            if (e.Button == MouseButtons.Right && m_sceneControl.Bubbles.Count > 0) //Clear all bubbles
            {
                m_sceneControl.Bubbles.Clear();
                Application.ActiveApplication.Output.Output("Stretching height increases or decreases 10m：This only works for layer items of Unique and Range Thematic maps, and two-dimensional vector layer of non-paste mode.");
            }
        }

        void m_sceneControl_MouseUp(object sender, MouseEventArgs e)
        {
            m_showBubble = false;
            if (m_bubble!=null) //Assign value to the bubble
            {
                m_bubble.Pointer = m_sceneControl.Scene.PixelToGlobe(e.Location);
            }
            if (m_sceneControl.Scene.FindSelection(true).Length ==0)   //If there is no selected object in scene, clear all bubble
            {
                m_sceneControl.Bubbles.Clear();
            }
        }

        /// <summary>
        /// When the Bubble is moved, the custom Bubble panel needs to follow it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
         void m_sceneControl_BubbleResize(object sender, BubbleEventArgs e)
         {
             if (m_customBubbleControl != null)
             {
                 Point point = new Point(e.Bubble.ClientLeft, e.Bubble.ClientTop);
                 m_customBubbleControl.Visible = false;
                 m_customBubbleControl.Location = point;
                 m_customBubbleControl.Visible = true;
             }
         }
        /// <summary>
        /// When closing Bubble, hide the custom Bubble panel
        /// </summary>
         void m_sceneControl_BubbleClose(object sender, BubbleEventArgs e)
         {
             if (m_customBubbleControl!=null)
             {
                 m_customBubbleControl.Visible = false;
             }
         }
        /// <summary>
        /// When initializing Bubble, put the custom contents to Bubble.
        /// </summary>
         void m_sceneControl_BubbleInitialize(object sender, BubbleEventArgs e)
         {
             if (m_customBubbleControl != null)
             {
                 Point point = new Point(e.Bubble.ClientLeft + 1, e.Bubble.ClientTop);
                 m_customBubbleControl.Location = point;
                 m_customBubbleControl.Visible = true;
             }
         }

        /// <summary>
        /// customize the Bubble content
        /// </summary>
        /// <param name="geometry"></param>
         void fillCustomBubbleControl(Selection3D selection)
         {
             if (selection.Layer is Layer3DDataset)
             {
                 Recordset recordset = selection.ToRecordset();
                 Geometry geometry = recordset.GetGeometry();
                 try
                 {
                     m_customBubbleControl.GeoTypeValue = geometry.Type.ToString();
                     m_customBubbleControl.LayerInfoValue = selection.Layer.Caption;

                     m_customBubbleControl.XValue = geometry.InnerPoint.X.ToString();
                     m_customBubbleControl.YValue = geometry.InnerPoint.Y.ToString();
                     m_customBubbleControl.ZValue = "0";
                     if (geometry is Geometry3D)
                     {
                         m_customBubbleControl.ZValue = (geometry as Geometry3D).Position.Z.ToString();
                     }
                     m_customBubbleControl.otherInfo = "Others:";
                     m_customBubbleControl.otherInfoValue = "……";

                     #region//Special layer
                     String alias = "@" + (selection.Layer as Layer3DDataset).Dataset.Datasource.Alias;
                     if (selection.Layer.DataName == "Ground" + alias)
                     {
                         m_customBubbleControl.otherInfo = "Ground Property:";
                         m_customBubbleControl.otherInfoValue = recordset.GetFieldValue("Ground Property") as String;
                     }
                     if (selection.Layer.DataName == "Tree" + alias)
                     {
                         m_customBubbleControl.otherInfo = "Varieties of trees:";
                         m_customBubbleControl.otherInfoValue = recordset.GetFieldValue("Varieties of trees") as String;
                     }
                     if (selection.Layer.DataName == "Building_1" + alias || selection.Layer.DataName == "Building_2" + alias)
                     {
                         m_customBubbleControl.otherInfo = "Wall Property:";
                         m_customBubbleControl.otherInfoValue = recordset.GetFieldValue("Wall Property") as String;
                     }
                     #endregion
                 }
                 catch (Exception ex)
                 {
                     SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
                 }
                 finally
                 {
                     if (recordset != null)
                     {
                         recordset.Dispose();
                     }
                     if (geometry != null)
                     {
                         geometry.Dispose();
                     }
                 }
             }
         }
    }
}
